#!/usr/bin/env python
# -*- coding: utf-8 -*-

version_info = (0, 1, 'a3')
__version__ = ".".join([str(x) for x in version_info])

from rdlmpy.client import RDLMClient
from rdlmpy.context import RDLMContextManager
from rdlmpy.lock import RDLMLock, RDLMActiveLock, RDLMWaitingLock
from rdlmpy.exceptions import RDLMException, RDLMLockWaitExceededException, RDLMLockDeletedException, RDLMLockServerException

__all__ = ['RDLMLock', 'RDLMActiveLock', 'RDLMWaitingLock', 'RDLMClient', 'RDLMContextManager', 'RDLMException', 'RDLMLockWaitExceededException', 'RDLMLockDeletedException', 'RDLMLockServerException']




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rdlm-py released under the MIT license.
# See the LICENSE file for more information.

import requests
import socket
import getpass
import os
import sys
from requests.auth import HTTPBasicAuth
import json
from rdlmpy.lock import RDLMLock
from rdlmpy.exceptions import RDLMLockWaitExceededException
from rdlmpy.exceptions import RDLMLockDeletedException
from rdlmpy.exceptions import RDLMLockServerException
from rdlmpy import __version__ as VERSION


class RDLMClient(object):
    '''
    Class which defines a client object
    '''

    _base_url = None
    _default_title = None
    _default_lifetime = None
    _default_wait = None

    def __init__(self, server="localhost", port=8888, default_title=None,
                 default_lifetime=300, default_wait=10):
        '''
        @summary: constructor
        @param server: rdlm server hostname
        @param port: rdlm server port
        @param default_title: default title for locks
        @param default_lifetime: default lifetime for locks (in seconds)
        @param default_wait: default wait time for locks (in seconds)
        @result: client object
        '''
        self._base_url = "http://%s:%i" % (server, port)
        if default_title:
            self._default_title = default_title
        else:
            self._default_title = "%s#%i(%s) with RDLMClient(%s) @%s" % (
                                  getpass.getuser(), os.getpid(), sys.argv[0],
                                  VERSION, socket.gethostname())
        self._default_lifetime = default_lifetime
        self._default_wait = default_wait

    def lock_acquire(self, resource_name, lifetime=None, wait=None,
                     title=None):
        '''
        @summary: acquires a lock
        @param resource_name: name of the resource to lock
        @param lifetime: lifetime for the lock (in seconds)
        @param wait: wait time for the lock (in seconds)
        @param title: title
        @result: lock url (if the lock is acquired)

        If the lock is not acquired, this function can raise :
        - a RDLMLockWaitExceededException: can't acquire the lock after
          "wait" seconds
        - a RDLMLockDeletedException: the request has been deleted by
          an admin request
        - a RDLMLockServerException: unknown error from the RDLM server
        '''
        lock_dict = {}
        lock_dict['lifetime'] = self._default_lifetime \
            if lifetime is None else lifetime
        lock_dict['wait'] = self._default_wait if wait is None else wait
        lock_dict['title'] = self._default_title if title is None else title
        lock_raw = json.dumps(lock_dict)
        r = requests.post("%s/locks/%s" % (self._base_url, resource_name), data=lock_raw)
        if r.status_code == 408:
            raise RDLMLockWaitExceededException()
        elif r.status_code == 409:
            raise RDLMLockDeletedException()
        elif r.status_code != 201 or not(r.headers['Location'].startswith('http://')):
            raise RDLMLockServerException()
        return r.headers['Location']

    def lock_release(self, lock_url):
        '''
        @summary: releases a lock
        @param lock_url: the lock url to release
        @result: True (if ok), False (else)

        The lock url is the return value of lock_acquire() method
        '''
        r = requests.delete(lock_url)
        return (r.status_code == 204)

    def lock_get(self, lock_url):
        '''
        @summary: gets informations about a lock
        @param lock_url: the lock url
        @result: informations dict (or None)
        '''
        r = requests.get(lock_url)
        try:
            if r.status_code == 200:
                return RDLMLock.factory(lock_url, r.content.decode('utf-8'))
        except:
            pass
        return None

    def resource_delete(self, resource_name, username=None, password=None):
        '''
        @summary: delete all locks on a resource
        @param resource_name: name of the resource
        @param username: admin http username
        @param password: admin http password
        @result: True if there were some locks on the resource, False else
        '''
        if username and password:
            auth = HTTPBasicAuth(username, password)
            r = requests.delete("%s/resources/%s" % (self._base_url, resource_name), auth=auth)
        else:
            r = requests.delete("%s/resources/%s" % (self._base_url, resource_name))
        return (r.status_code == 204)

    def resource_delete_all(self, username=None, password=None):
        '''
        @summary: delete all locks on all resources
        @param username: admin http username
        @param password: admin http password
        @result: True if ok, False else
        '''
        if username and password:
            auth = HTTPBasicAuth(username, password)
            r = requests.delete("%s/resources" % self._base_url, auth=auth)
        else:
            r = requests.delete("%s/resources" % self._base_url)
        return (r.status_code == 204)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rdlm-py released under the MIT license.
# See the LICENSE file for more information.

import json
import datetime


def iso8601_to_datetime(iso8601_string):
    return datetime.datetime.strptime(iso8601_string[0:19], "%Y-%m-%dT%H:%M:%S")


class RDLMLock(object):
    '''
    Class which defines a lock object
    '''

    url = None
    title = None
    uid = None
    lifetime = None
    wait = None

    def __init__(self, url):
        self.url = url

    @staticmethod
    def factory(url, get_request_output):
        res = None
        try:
            tmp = json.loads(get_request_output)
            if tmp['active']:
                res = RDLMActiveLock(url)
                res.active_since = iso8601_to_datetime(tmp['active_since'])
                res.active_expires = iso8601_to_datetime(tmp['active_expires'])
            else:
                res = RDLMWaitingLock(url)
                res.wait_since = iso8601_to_datetime(tmp['active_since'])
                res.wait_expires = iso8601_to_datetime(tmp['active_expires'])
            res.title = tmp['title']
            res.uid = tmp['uid']
            res.lifetime = tmp['lifetime']
            res.wait = tmp['wait']
        except:
            pass
        return res


class RDLMActiveLock(RDLMLock):
    active_since = None
    active_expires = None

    @property
    def active(self):
        return True


class RDLMWaitingLock(RDLMLock):
    wait_since = None
    wait_expires = None

    @property
    def active(self):
        return False



import unittest
from rdlmpy import RDLMContextManager
from httpretty import HTTPretty, httprettified
from rdlmpy import RDLMLockWaitExceededException

class SpecialException(Exception):

    pass

class TestContext(unittest.TestCase):

    context = None
    server = "localhost"
    port = 8888
    baseurl = None
    resource = "foo"

    def setUp(self):
        self.context = RDLMContextManager(self.resource, server=self.server, port=self.port)
        self.baseurl = "http://%s:%i" % (self.server, self.port)
        self.assertFalse(self.context is None)

    def tearDown(self):
        pass

    @httprettified
    def test_enterexit(self):
        url = "%s/locks/%s" % (self.baseurl, self.resource)
        lock_url = "%s/ff14608f6ab342f0bb2a86d551d42a8c" % url
        HTTPretty.register_uri(HTTPretty.POST, url, status=201, location=lock_url)
        HTTPretty.register_uri(HTTPretty.DELETE, lock_url, status=204)
        result = False
        with self.context as c:
            self.assertFalse(c is None)
            result = True
        self.assertTrue(result)

    @httprettified
    def test_enterexit408(self):
        url = "%s/locks/%s" % (self.baseurl, self.resource)
        HTTPretty.register_uri(HTTPretty.POST, url, status=408)
        try:
            with self.context as c:
                raise Exception("no exception raised")
        except RDLMLockWaitExceededException:
            pass

    @httprettified
    def test_enterexit_exception(self):
        def _delete_callback(method, uri, headers):
            self.delete_flag = True
            return "coucou"
        self.delete_flag = False
        url = "%s/locks/%s" % (self.baseurl, self.resource)
        lock_url = "%s/ff14608f6ab342f0bb2a86d551d42a8c" % url
        HTTPretty.register_uri(HTTPretty.POST, url, status=201, location=lock_url)
        HTTPretty.register_uri(HTTPretty.DELETE, lock_url, body=_delete_callback, status=204)
        try:
            with self.context as c:
                raise SpecialException("no exception raised")
        except SpecialException:
            pass
        self.assertTrue(self.delete_flag)

if __name__ == '__main__':
    unittest.main()



import unittest
from rdlmpy import RDLMClient
from rdlmpy import RDLMLockWaitExceededException, RDLMLockDeletedException, RDLMLockServerException
from httpretty import HTTPretty, httprettified
import base64
import os

class TestClient(unittest.TestCase):

    client = None
    server = "localhost"
    port = 8888
    baseurl = None
    resource = "foo"

    def setUp(self):
        self.baseurl = "http://%s:%i" % (self.server, self.port)
        self.client = RDLMClient(server=self.server, port=self.port)
        self.assertFalse(self.client is None)

    def tearDown(self):
        pass

    def _make_lock_object(self):
        url = "%s/locks/%s" % (self.baseurl, self.resource)
        lock_url = "%s/ff14608f6ab342f0bb2a86d551d42a8c" % url
        HTTPretty.register_uri(HTTPretty.POST, url, status=201, location=lock_url)
        r = self.client.lock_acquire(self.resource)
        self.assertFalse(r is None)
        return (r, lock_url)

    @httprettified
    def test_acquire(self):
        (lock_u, lock_url) = self._make_lock_object()
        self.assertEqual(lock_u, lock_url)

    @httprettified
    def _test_acquire_exception(self, status_code, exception):
        url = "%s/locks/%s" % (self.baseurl, self.resource)
        if status_code == 201:
            lock_url = "%s/ff14608f6ab342f0bb2a86d551d42a8c" % url
            HTTPretty.register_uri(HTTPretty.POST, url, status=status_code, location=lock_url)
        else:
            HTTPretty.register_uri(HTTPretty.POST, url, status=status_code)
        try:
            self.client.lock_acquire(self.resource)
            raise Exception("no exception raised")
        except exception:
            pass

    @httprettified
    def test_acquire_408(self):
        self._test_acquire_exception(408, RDLMLockWaitExceededException)

    @httprettified
    def test_acquire_500(self):
        self._test_acquire_exception(500, RDLMLockServerException)

    @httprettified
    def test_acquire_409(self):
        self._test_acquire_exception(409, RDLMLockDeletedException)

    @httprettified
    def test_release(self):
        (lock_u, lock_url) = self._make_lock_object()
        self.assertEqual(lock_u, lock_url)
        HTTPretty.register_uri(HTTPretty.DELETE, lock_url, status=204)
        f = self.client.lock_release(lock_url)
        self.assertTrue(f)

    @httprettified
    def test_delete_all_resources(self):
        HTTPretty.register_uri(HTTPretty.DELETE, "%s/resources" % self.baseurl, status=401)
        f = self.client.resource_delete_all()
        self.assertFalse(f)
        self.assertFalse('Authorization' in HTTPretty.last_request.headers)
        HTTPretty.register_uri(HTTPretty.DELETE, "%s/resources" % self.baseurl, status=204)
        f = self.client.resource_delete_all("foo", "bar")
        self.assertTrue(f)
        self.assertTrue('Authorization' in HTTPretty.last_request.headers)
        b64 = base64.standard_b64encode(b"foo:bar")
        expected = b"Basic " + b64
        self.assertEqual(HTTPretty.last_request.headers['Authorization'].encode('ascii'), expected)

    @httprettified
    def test_delete_resource(self):
        HTTPretty.register_uri(HTTPretty.DELETE, "%s/resources/foo" % self.baseurl, status=401)
        f = self.client.resource_delete("foo")
        self.assertFalse(f)
        self.assertFalse('Authorization' in HTTPretty.last_request.headers)
        HTTPretty.register_uri(HTTPretty.DELETE, "%s/resources/foo" % self.baseurl, status=204)
        f = self.client.resource_delete("foo", username="foo", password="bar")
        self.assertTrue(f)
        self.assertTrue('Authorization' in HTTPretty.last_request.headers)
        b64 = base64.standard_b64encode(b"foo:bar")
        expected = b"Basic " + b64
        self.assertEqual(HTTPretty.last_request.headers['Authorization'].encode('ascii'), expected)

    @httprettified
    def test_get(self):
        lock_url = "%s/locks/foo/94e4458bad8248828213275ae0b17eae" % self.baseurl
        HTTPretty.register_uri(HTTPretty.GET, lock_url, status=404)
        f = self.client.lock_get(lock_url)
        self.assertTrue(f is None)
        json_file = os.path.join(os.path.dirname(__file__), "lock1.json")
        with open(json_file, "r") as f:
            body = f.read()
        HTTPretty.register_uri(HTTPretty.GET, lock_url, status=200, body=body)
        f = self.client.lock_get(lock_url)
        self.assertFalse(f is None)
        self.assertTrue(f.active)
        self.assertEqual(f.title, "test title")
        self.assertEqual(f.uid, "94e4458bad8248828213275ae0b17eae")
        self.assertEqual(f.lifetime, 300)
        self.assertEqual(f.wait, 10)
        self.assertEqual(f.active_since.isoformat(), "2013-03-02T22:00:05")
        self.assertEqual(f.active_expires.isoformat(), "2013-03-02T22:05:05")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rdlm-py released under the MIT license.
# See the LICENSE file for more information.


class RDLMException(Exception):
    '''
    Abstract class to represent a generic RDLM exception
    '''
    pass


class RDLMLockWaitExceededException(RDLMException):
    '''
    Exception: the lock is not acquired after wait timeout
    '''
    pass


class RDLMLockDeletedException(RDLMException):
    '''
    Exception: the lock has been deleted by an admin request
    '''
    pass


class RDLMLockServerException(RDLMException):
    '''
    Unknown exception from the RDLM server
    '''
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rdlm-py released under the MIT license.
# See the LICENSE file for more information.

from rdlmpy.client import RDLMClient


class RDLMContextManager(object):
    '''
    Class which defines a ContextManager to get a convenient way to use RDLMClient with
    the "with" statement
    '''

    __client = None
    __resource_name = None
    __lock = None

    def __init__(self, resource_name, server="localhost", port=8888, lifetime=300, wait=10,
                 title=None):
        self.__client = RDLMClient(server=server, port=port, default_lifetime=lifetime,
                                   default_wait=wait, default_title=title)
        self.__resource_name = resource_name

    def __enter__(self):
        self.__lock = self.__client.lock_acquire(self.__resource_name)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.__lock:
            self.__client.lock_release(self.__lock)
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rdlm-py released under the MIT license.
# See the LICENSE file for more information.

from setuptools import setup, find_packages
import rdlmpy

DESCRIPTION = "rdlm-py is a python client for RDLM (Restful Distributed Lock Manager)"
try:
    with open('PIP.rst') as f:
        LONG_DESCRIPTION = f.read()
except IOError:
    LONG_DESCRIPTION = DESCRIPTION

with open('pip-requirements.txt') as reqs:
    install_requires = [
        line for line in reqs.read().split('\n') if (line and not
                                                     line.startswith('--'))]

setup(
    name='rdlm-py',
    version=rdlmpy.__version__,
    author="Fabien MARTY",
    author_email="fabien.marty@gmail.com",
    url="https://github.com/thefab/rdlm-py",
    packages=find_packages(),
    license='MIT',
    download_url='https://github.com/thefab/rdlm-py',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Utilities',
        'Topic :: System :: Distributed Computing',
        'Topic :: Software Development',
    ]
)
